<?php


/**
* Authentication module
*
*/
class AuthModule
{
	private $db;

	/**
	 * Constructor
	 *
	 * @param PEAR::DB $db db object
	 */
	public function __construct(&$db)
	{
		$this->db = $db;
	}

	/**
	 * Returns id of actually logged user
	 *
	 * @return int
	 */
	public function get_id()
	{
		return $_SESSION['auth_user'];
	}

	/**
	 * Sets actually logged user id
	 *
	 * @param $id id of current user
	 */
	public function set_id($id)
	{
		$_SESSION['auth_user'] = $id;
	}

	/**
	 * Returns login name of actual user or null if no user is logged in
	 *
	 * @return string login of user or null
	 */
	public function get_login()
	{
		$id = $this->get_id();
		if (!is_numeric($id))
			return null;
		return $this->get_login_by_id($id);
	}

	/**
	 * Returns login name of user specified by id
	 *
	 * @param int $id id of user
	 * @return string login of user or null
	 */
	public function get_login_by_id($id)
	{
		if (!is_numeric($id))
			return null;
		$login = $this->db->getOne("SELECT login FROM `submitter` WHERE id=?", $id);
		if (PEAR::isError($login))
			throw new SAuthException($login->getMessage());
		return $login;
	}

	/**
	 * Returns true if current user has admin rights, false if not.
	 * Otherwise returns null if no current user is logged in.
	 *
	 * @return bool
	 */
	public function is_admin()
	{
		$level = $this->get_level();
		if ($level == null)
			return null;
		return $level == 0 ? true : false;
	}

	/**
	 * Returns user level.
	 * 0 ... admin (all privilegs)
	 * 1 ... submitter (can search, download documents and add publications)
	 * 2 ... ordinary user (can search and download documents)
	 *
	 * @return int
	 */
	public function get_level()
	{
		$id = $this->get_id();
		if (!is_numeric($id))
			return null;
		$level = $this->db->getOne("SELECT level FROM `submitter` WHERE id=?", $id);
		if (PEAR::isError($level))
			throw new SAuthException($level->getMessage());
		return $level;
	}

	/**
	 * Returns textual explanation of user rights level
	 *
	 * @return string
	 */
	public function get_role_name()
	{
		switch ($this->get_level())
		{
			case 0:
				return 'administrator';
				break;
			case 1:
				return 'submitter';
				break;
			case 2:
				return 'user';
				break;
			default:
				return null;
		}
	}

	/**
	 * Checks if given login name and password fit together
	 *
	 * @param string $login user login name
	 * @param string $pass user password
	 * @return false if login failed, id of user if login succeded
	 */
	public function check_user($login, $pass)
	{
		$fp = md5(trim($pass));
		$res = $this->db->query("SELECT id FROM `submitter` WHERE login='$login' AND pass='$fp'");
		if ($res->numRows() == 0)
			return false;
		if ($res->numRows() > 1)
			throw new SAuthException("Fatal error of auth. module, please contact administrator for help.");
		$row = $res->fetchRow(DB_FETCHMODE_ORDERED);
		return $row[0];
	}

	/**
	 * Check user credentials, if ok set user as authorized
	 *
	 * @param string $login
	 * @param string $pass
	 * @return bool true if passed false if rejected
	 */
	public function login($login, $pass)
	{
		$id = $this->check_user($login, $pass);
		if ($id == false)
			return false;
		$_SESSION['auth_user'] = $id;
		return true;
	}

	/**
	 * Log out
	 *
	 */
	public function logout()
	{
		unset($_SESSION['auth_user']);
	}


	/**
	 * Creates user with specified credentials. If the same user already exists throws an Exception.
	 *
	 * @param string $name
	 * @param string $email
	 * @param string $login
	 * @param string $pass
	 * @param int $level optional parameter permition level
	 *
	 * @return id of created user
	 */
	public function create_user($name, $surname, $email, $login, $pass, $level=2)
	{
		$res = $this->db->query("SELECT id FROM `submitter` WHERE login='$login'");
		if (PEAR::isError($res))
			throw new SAuthException($res->getMessage());
		if ($res->numRows() > 0)
			throw new Exception("User '$login' already exists.");
		$res = $this->db->query("SELECT id FROM `submitter` WHERE email='$email'");
		if (PEAR::isError($res))
			throw new SAuthException($res->getMessage());
		if ($res->numRows() > 0)
			throw new Exception("User with email '$email' already exists.");
		$hash = md5($pass);
		$res = $this->db->query("INSERT INTO `submitter` (name, surname, email, login, pass, level)
				VALUES ('$name', '$surname', '$email', '$login', '$hash', '$level')");
		if (PEAR::isError($res))
			throw new SAuthException($res->getMessage());
		return last_insert_id($this->db);
	}

	/**
	 * Deletes user
	 *
	 * @param string $login
	 * @return int number of deleted users (usualy 1 or 0)
	 */
	public function delete_user($login)
	{   if (confirm("Are you sure you want to delete")) {
		$res = $this->db->query("DELETE FROM `submitter` WHERE login='$login'");
		if (PEAR::isError($res))
			throw new SAuthException($res->getMessage());
		return $this->db->affectedRows();
	   }
	}

	/**
	 * Gets user credentials as an associative array
	 *
	 * @return array
	 */
	public function get_credentials()
	{
		$id = $this->get_id();
		if (!is_numeric($id))
			throw new SAuthException("No user is logged in, can't get any credentials.");
		$res = $this->db->query("SELECT name, surname, email, login FROM `submitter` WHERE id=?", $id);
		if (PEAR::isError($res))
			throw new SAuthException($res->getMessage());
		$row = $res->fetchRow(DB_FETCHMODE_ASSOC);
		return $row;
	}

	/**
	 * Set new password for current user
	 *
	 * @param string $pass new password, trimmed
	 * @param string $current_pass current password, trimmrd
	 */
	public function set_password($pass, $current_pass)
	{
		$id = $this->get_id();
		if (!is_numeric($id))
			throw new SAuthException("No user is logged in, can't change password.");
		if (strlen($pass) == 0)
			throw new Exception("Password is empty.");
		$cp = $this->db->getOne("SELECT pass FROM `submitter` WHERE id=?", $id);
		if (PEAR::isError($cp))
			throw new SAuthException($cp->getMessage());
		if ($cp != md5($current_pass))
			throw new Exception("Current password you entered is wrong.");
		$res = $this->db->query('UPDATE `submitter` SET pass=? WHERE id=?', array(md5($pass), $id));
		if (PEAR::isError($id))
			throw new SAuthException($res->getMessage());
	}

	/**
	 * Fetches request on generating new password
	 *
	 * @param string $email valid email address
	 */
	public function forgotten_pass_req($email)
	{
		$id = $this->db->getOne("SELECT id FROM `submitter` WHERE email='$email'");
		if (PEAR::isError($id))
			throw new SAuthException($res->getMessage());
		if (empty($id))
			throw new Exception("Email '$email' is not registered.");

		$this->db->autoCommit(false);
		$uid = uniqid();
		$uid_hash = md5($uid);
		$res = $this->db->query('DELETE FROM `retrieve` WHERE submitter_id=?', $id); //remove possible duplicate entries
		if (PEAR::isError($res))
		{
			$this->db->rollback();
			throw new SAuthException($res->getMessage());
		}
		$res = $this->db->query("INSERT INTO `retrieve` (submitter_id, uid_hash) VALUES ('$id', '$uid_hash')");
		if (PEAR::isError($res))
		{
			$this->db->rollback();
			throw new SAuthException($res->getMessage());
		}
		global $account_url;
		$subject = 'Password retrieval';
		$message = "Dear user, \n
this message was automatically sent as a reaction on your request \n
for password retrieval. \n
Please don't reply. \n
If you didn't initiate this password retrieval - delete this email. \n
------------------- \n
In other case, confirm your request by clicking on the following link: \n
$account_url"."index.php?form=retrieved_password&amp;submitter_id=$id&amp;uid=$uid \n
With other questions or suggestions don't hesitate to contact sir Petr Fišer by mail fiserp@fel.cvut.cz \n\n
Thanks for your interest.";
		$eol="\n";
		$headers = "From: Database of publiactions <no-reply@vlsi.felk.cvut.cz>".$eol;
		$headers .= "X-Sender: <no-reply@vlsi.felk.cvut.cz>".$eol;
		$headers .= "Return-Path: <no-reply@vlsi.felk.cvut.cz>".$eol;
		$headers .= "Content-type: text/plain; charset=utf-8".$eol;

		$sent = @mail($email, $subject, $message, $headers);
		if (!$sent)
		{
			$this->db->rollback();
			throw new SAuthException('Function mail() was unable to send the message.');
		}
		//print nl2br($message);
		$this->db->commit();
	}

	/**
	 * Sets new password for given user id and returns it by out parameter.
	 * Parameter $uid must match with the record in database.
	 *
	 * @param int $id user (submitter) id
	 * @param string $uid unique id - protect against some attacks
	 * @param string $login out parameter - will contain login of user
	 * @param string $pass out parameter - will contain newly generated password
	 */
	public function reset_password($id, $uid, &$login, &$pass)
	{
		$this->db->autoCommit(false);
		$found = $this->db->getOne("SELECT COUNT(*) FROM `retrieve`
			WHERE submitter_id=? AND uid_hash=?", array($id, md5($uid)));
		if ($found != 1)
			throw new Exception("Password couldn't be retrieved, check if the confirming html link, what
				you have recieved is copied propperly into the browser. Check if
				you have't displayed this page recently, retrieved password can be displayed only once.");
		//assign to the out parameters
		$login = $this->get_login_by_id($id);
		$pass = substr(uniqid(), 7, 6);
		$res = $this->db->query("UPDATE `submitter` SET pass=? WHERE id=?", array(md5($pass), $id));
		if (PEAR::isError($res))
		{
			$this->db->rollback();
			throw new SAuthException($res->getMessage());
		}
		$res = $this->db->query("DELETE FROM `retrieve` WHERE submitter_id=?", $id);
		if (PEAR::isError($res))
		{
			$this->db->rollback();
			throw new SAuthException($res->getMessage());
		}
		$this->db->commit();
	}

}

?>